@@ -25,7 +25,8 @@
 #include <klocalizedstring.h>
 #include <QFile>
 #include <QStandardPaths>
-#include <QDebug>
+#include "libdiscover_debug.h"
+#include <utils.h>
 
 Category::Category(QSet<QString> pluginName, QObject* parent)
         : QObject(parent)
@@ -42,7 +43,9 @@ Category::Category(const QString& name, const QString& iconName, const QVector<Q
     , m_subCategories(subCategories)
     , m_plugins(pluginName)
     , m_isAddons(isAddons)
-{}
+{
+    setObjectName(m_name);
+}
 
 Category::~Category() = default;
 
@@ -52,13 +55,14 @@ void Category::parseData(const QString& path, const QDomNode& data)
     {
         if(!node.isElement()) {
             if(!node.isComment())
-                qWarning() << "unknown node found at " << QStringLiteral("%1:%2").arg(path).arg(node.lineNumber());
+                qCWarning(LIBDISCOVER_LOG) << "unknown node found at " << QStringLiteral("%1:%2").arg(path).arg(node.lineNumber());
             continue;
         }
         QDomElement tempElement = node.toElement();
 
         if (tempElement.tagName() == QLatin1String("Name")) {
             m_name = i18nc("Category", tempElement.text().toUtf8().constData());
+            setObjectName(m_name);
         } else if (tempElement.tagName() == QLatin1String("Menu")) {
             m_subCategories << new Category(m_plugins, this);
             m_subCategories.last()->parseData(path, node);
@@ -67,7 +71,7 @@ void Category::parseData(const QString& path, const QDomNode& data)
             if (m_decoration.isRelative()) {
                 m_decoration = QUrl::fromLocalFile(QStandardPaths::locate(QStandardPaths::GenericDataLocation, QStringLiteral("discover/") + tempElement.text()));
                 if (m_decoration.isEmpty())
-                    qWarning() << "couldn't find category decoration" << tempElement.text();
+                    qCWarning(LIBDISCOVER_LOG) << "couldn't find category decoration" << tempElement.text();
             }
         } else if (tempElement.tagName() == QLatin1String("Addons")) {
             m_isAddons = true;
@@ -107,7 +111,7 @@ QVector<QPair<FilterType, QString> > Category::parseIncludes(const QDomNode &dat
         } else if (tempElement.tagName() == QLatin1String("PkgName")) {
             filter.append({ PkgNameFilter, tempElement.text() });
         } else {
-            qWarning() << "unknown" << tempElement.tagName();
+            qCWarning(LIBDISCOVER_LOG) << "unknown" << tempElement.tagName();
         }
         node = node.nextSibling();
     }
@@ -150,55 +154,85 @@ bool Category::categoryLessThan(Category *c1, const Category *c2)
     return (!c1->isAddons() && c2->isAddons()) || (c1->isAddons()==c2->isAddons() && QString::localeAwareCompare(c1->name(), c2->name()) < 0);
 }
 
-//TODO: maybe it would be interesting to apply some rules to a said backend...
+static bool isSorted(const QVector<Category*>& vector)
+{
+    Category *last = nullptr;
+    for(auto a: vector) {
+        if (last && !Category::categoryLessThan(last, a))
+            return false;
+        last = a;
+    }
+    return true;
+}
+
+void Category::sortCategories(QVector<Category *>& cats)
+{
+    std::sort(cats.begin(), cats.end(), &categoryLessThan);
+    for(auto cat: cats) {
+        sortCategories(cat->m_subCategories);
+    }
+    Q_ASSERT(isSorted(cats));
+}
+
 void Category::addSubcategory(QVector< Category* >& list, Category* newcat)
 {
-    Q_FOREACH (Category* c, list) {
-        if(c->name() == newcat->name()) {
-            if(c->icon() != newcat->icon()
-                || c->m_andFilters != newcat->m_andFilters
-                || c->m_isAddons != newcat->m_isAddons
-            )
-            {
-                qWarning() << "the following categories seem to be the same but they're not entirely"
-                    << c->name() << newcat->name() << "--"
-                    << c->andFilters() << newcat->andFilters() << "--"
-                    << c->isAddons() << newcat->isAddons();
-                break;
-            } else {
-                c->m_orFilters += newcat->orFilters();
-                c->m_notFilters += newcat->notFilters();
-                c->m_plugins.unite(newcat->m_plugins);
-                Q_FOREACH (Category* nc, newcat->subCategories()) {
-                    nc->setParent(c);
-                    addSubcategory(c->m_subCategories, nc);
-                }
-                delete newcat;
-                return;
-            }
-        }
+    Q_ASSERT(isSorted(list));
+
+    auto it = std::lower_bound(list.begin(), list.end(), newcat, &categoryLessThan);
+    if (it == list.end()) {
+        list << newcat;
+        return;
     }
-    for(auto it = list.begin(), itEnd = list.end(); it!=itEnd; ++it) {
-        if (!categoryLessThan(*it, newcat)) {
-            list.insert(it, newcat);
+
+    auto c = *it;
+    if(c->name() == newcat->name()) {
+        if(c->icon() != newcat->icon()
+            || c->m_andFilters != newcat->m_andFilters
+            || c->m_isAddons != newcat->m_isAddons
+        )
+        {
+            qCWarning(LIBDISCOVER_LOG) << "the following categories seem to be the same but they're not entirely"
+                << c->name() << newcat->name() << "--"
+                << c->andFilters() << newcat->andFilters() << "--"
+                << c->isAddons() << newcat->isAddons();
+        } else {
+            c->m_orFilters += newcat->orFilters();
+            c->m_notFilters += newcat->notFilters();
+            c->m_plugins.unite(newcat->m_plugins);
+            Q_FOREACH (Category* nc, newcat->subCategories()) {
+                addSubcategory(c->m_subCategories, nc);
+            }
             return;
         }
     }
-    list << newcat;
+
+    list.insert(it, newcat);
+    Q_ASSERT(isSorted(list));
+}
+
+bool Category::blacklistPluginsInVector(const QSet<QString>& pluginNames, QVector<Category *>& subCategories)
+{
+    bool ret = false;
+    for(QVector<Category*>::iterator it = subCategories.begin(), itEnd = subCategories.end(); it!=itEnd; ) {
+        if ((*it)->blacklistPlugins(pluginNames)) {
+            delete *it;
+            it = subCategories.erase(it);
+            ret = true;
+        } else
+            ++it;
+    }
+    return ret;
 }
 
 bool Category::blacklistPlugins(const QSet<QString>& pluginNames)
 {
-    for(QVector<Category*>::iterator it = m_subCategories.begin(), itEnd = m_subCategories.end(); it!=itEnd; ) {
-        if ((*it)->blacklistPlugins(pluginNames)) {
-            delete *it;
-            it = m_subCategories.erase(it);
-        } else
-            ++it;
+    if (m_plugins.subtract(pluginNames).isEmpty()) {
+        return true;
     }
-    m_plugins.subtract(pluginNames);
 
-    return m_plugins.isEmpty();
+    if (blacklistPluginsInVector(pluginNames, m_subCategories))
+        Q_EMIT subCategoriesChanged();
+    return false;
 }
 
 QUrl Category::decoration() const
@@ -214,12 +248,7 @@ QUrl Category::decoration() const
 
 QVariantList Category::subCategoriesVariant() const
 {
-    QVariantList ret;
-    ret.reserve(m_subCategories.count());
-    for(Category* cat : m_subCategories) {
-        ret.append(QVariant::fromValue<QObject*>(cat));
-    }
-    return ret;
+    return kTransform<QVariantList>(m_subCategories, [](Category* cat){ return QVariant::fromValue<QObject*>(cat); });
 }
 
 bool Category::matchesCategoryName(const QString& name) const
@@ -240,7 +269,7 @@ bool Category::contains(Category* cat) const
 bool Category::contains(const QVariantList& cats) const
 {
     bool ret = false;
-    for(auto itCat : cats) {
+    for(const auto &itCat : cats) {
         if (contains(qobject_cast<Category*>(itCat.value<QObject*>()))) {
             ret = true;
             break;
